// (C) Copyright 2009 Andreas Kloeckner

// Use, modification and distribution is subject to the Boost Software
// License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

//  Authors: Andreas Kloeckner

#ifndef BOOSTMPI_ENVIRONMENT_HPP
#define BOOSTMPI_ENVIRONMENT_HPP

#include <boost/python.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/mpi.hpp>




namespace boost { namespace mpi { namespace python {

boost::shared_ptr<environment> current_environment();





class explicit_environment_dependent
{
  private:
    boost::shared_ptr<environment> m_ward_environment;

  public:
    void acquire_environment()
    {
      m_ward_environment = current_environment();
      if (m_ward_environment.get() == 0)
        throw std::runtime_error("boostmpi::explicit_environment_dependent: "
            "no active environment - MPI not initialized?");
    }

    void release_environment()
    {
      m_ward_environment.reset();
    }

    boost::shared_ptr<environment> get_environment()
    {
      return m_ward_environment;
    }
};




class environment_dependent : public explicit_environment_dependent
{
  private:
    boost::shared_ptr<environment> m_ward_environment;

  public:
    environment_dependent()
    { acquire_environment(); }
};



// environment_dependent wrappers of existing objects -------------------------
struct py_communicator : public communicator, private environment_dependent
{
  py_communicator()
  { }

  py_communicator(const MPI_Comm &com, comm_create_kind ck)
    : communicator(com, ck)
  { }

  py_communicator(const communicator &com, const boost::mpi::group &grp)
    : communicator(com, grp)
  { }
};

struct py_request : public request, boost::mpi::python::environment_dependent
{
  py_request()
  { }
  py_request(const request &r)
    : request(r)
  { }
};

struct py_status : public status, boost::mpi::python::environment_dependent
{
  py_status()
  { }
  py_status(const status &s)
    : status(s)
  { }
};

class py_timer : public timer, boost::mpi::python::environment_dependent
{
};

} } }

#endif
